@@ -21,11 +21,12 @@ class WorkerServer(BaseServer):
     public_ip = None
     local_ip = None
     network_information = {}
-    simultaneous_jobs = 100
-    doSomethingCallLater = None
+    simultaneous_jobs = 20
     jobs_complete = 0
     job_queue = []
     job_queue_a = job_queue.append
+    jobsloop = None
+    pending_dequeue = False
     
     def __init__(self,
             aws_access_key_id,
@@ -130,20 +131,21 @@ class WorkerServer(BaseServer):
         yield self.chan.queue_bind(
             queue=self.amqp_queue,
             exchange=self.amqp_exchange)
-        d = self.chan.basic_consume(queue=self.amqp_queue,
+        yield self.chan.basic_consume(queue=self.amqp_queue,
             no_ack=False,
             consumer_tag="awspider_consumer")
-        d.addCallback(self.dequeue)
         self.queue = yield self.conn.queue("awspider_consumer")
         yield BaseServer.start(self)
         self.jobsloop = task.LoopingCall(self.executeJobs)
         self.jobsloop.start(1)
+        LOGGER.info('Starting dequeueing thread...')
+        deferToThread(self.dequeue)
     
     @inlineCallbacks
     def shutdown(self):
-        LOGGER.debug("Closting connection")
+        LOGGER.debug("Closing connection")
         try:
-            self.doSomethingCallLater.cancel()
+            self.jobsloop.cancel()
         except:
             pass
         # Shut things down
@@ -151,66 +153,78 @@ class WorkerServer(BaseServer):
         yield self.chan.channel_close()
         chan0 = yield self.conn.channel(0)
         yield chan0.connection_close()
+        #close mysql connection
+        #close memcache connection
         
-    def dequeue(self, consumer=None):
-        LOGGER.info('Starting Dequeuing Thread...')
-        deferToThread(self._dequeue)
-    
-    def _dequeue(self):
-        if len(self.active_jobs) <= self.amqp_prefetch_count:
+    def dequeue(self):
+        LOGGER.debug('Pending Deuque: %s / Completed Jobs: %d / Queued Jobs: %d / Active Jobs: %d' % (self.pending_dequeue, self.jobs_complete, len(self.job_queue), len(self.active_jobs)))
+        if len(self.job_queue) <= self.amqp_prefetch_count and not self.pending_dequeue:
+            self.pending_dequeue = True
+            LOGGER.debug('Fetching from queue')
             d = self.queue.get()
             d.addCallback(self._dequeueCallback)
-            d.addErrback(self.workerError)
+            d.addErrback(self._dequeueErrback)
         else:
-            reactor.callLater(1, self._dequeue)
-    
+            reactor.callLater(1, self.dequeue)
+            
+    def _dequeueErrback(self, error):
+        LOGGER.error('Dequeue Error: %s' % error)
+        self.pending_dequeue = False
+        reactor.callLater(0, self.dequeue)
+        
     def _dequeueCallback(self, msg):
+        LOGGER.debug('fetched msg from queue: %s' % repr(msg))
         # Get the hex version of the UUID from byte string we were sent
         uuid = UUID(bytes=msg.content.body).hex
         d = self.getJob(uuid, msg.delivery_tag)
         d.addCallback(self._dequeueCallback2, msg)
-        d.addErrback(self.workerError)
+        d.addErrback(self._dequeueErrback, 'Dequeue Callback')
     
     def _dequeueCallback2(self, job, msg):
-        if job:
-            # Load custom function.
-            if job['function_name'] in self.functions:
-                job['exposed_function'] = self.functions[job['function_name']]
-            else:
-                LOGGER.error("Could not find function %s." % job['function_name'])
-            LOGGER.info('Pulled job off of AMQP queue')
+        # Load custom function.
+        if job['function_name'] in self.functions:
+            job['exposed_function'] = self.functions[job['function_name']]
+            LOGGER.debug('Pulled job off of AMQP queue')
             job['kwargs'] = self.mapKwargs(job)
             job['delivery_tag'] = msg.delivery_tag
             self.job_queue_a(job)
-        self._dequeue()
-    
+        else:
+            LOGGER.error("Could not find function %s." % job['function_name'])
+        self.pending_dequeue = False
+        reactor.callLater(0, self.dequeue)
+        
     def executeJobs(self):
         while len(self.job_queue) > 0 and len(self.active_jobs) < self.simultaneous_jobs:
             job = self.job_queue.pop(0)
             exposed_function = job["exposed_function"]
             kwargs = job["kwargs"]
             function_name = job["function_name"]
-            uuid = job["uuid"]
+            if job.has_key('uuid'):
+                uuid = job["uuid"]
+            else:
+                # assign a temp uuid
+                uuid = UUID(bytes=msg.content.body).hex
             d = self.callExposedFunction(
                 exposed_function["function"], 
                 kwargs, 
                 function_name, 
                 uuid=uuid)
             d.addCallback(self._executeJobCallback, job)
-            d.addErrback(self.workerError)
+            d.addErrback(self.workerErrback, 'Execute Jobs')
         
     def _executeJobCallback(self, data, job):
         self.chan.basic_ack(delivery_tag=job['delivery_tag'])
         self.jobs_complete += 1
-        LOGGER.info('Completed %d jobs' % self.jobs_complete)
+        LOGGER.debug('Completed Jobs: %d / Queued Jobs: %d / Active Jobs: %d' % (self.jobs_complete, len(self.job_queue), len(self.active_jobs)))
         
-    def workerError(self, error):
-        LOGGER.error('Worker Error: %s' % str(error))
+    def workerErrback(self, error, function_name='Worker'):
+        LOGGER.error('%s Error: %s' % (function_name, str(error)))
+        self.pending_dequeue = False
     
     def getJob(self, uuid, delivery_tag):
         d = self.memc.get(uuid)
         d.addCallback(self._getJobCallback, uuid, delivery_tag)
-        d.addErrback(self.workerError)
+        d.addErrback(self.workerErrback, 'Get Job')
         return d
     
     def _getJobCallback(self, account, uuid, delivery_tag):
@@ -220,7 +234,7 @@ class WorkerServer(BaseServer):
             sql = "SELECT account_id, type FROM spider_service WHERE uuid = '%s'" % uuid
             d = self.mysql.runQuery(sql)
             d.addCallback(self.getAccountMySQL, uuid, delivery_tag)
-            d.addErrback(self.workerError)
+            d.addErrback(self.workerErrback, 'Get Job Callback')
             return d
         else:
             LOGGER.debug('Found uuid in memcached: %s' % uuid)
@@ -232,9 +246,9 @@ class WorkerServer(BaseServer):
             sql = "SELECT * FROM content_%saccount WHERE account_id = %d" % (account_type, spider_info[0]['account_id'])
             d = self.mysql.runQuery(sql)
             d.addCallback(self.createJob, spider_info, uuid)
-            d.addErrback(self.workerError)
+            d.addErrback(self.workerErrback, 'Get MySQL Account')
             return d
-        LOGGER.info('No spider_info given for uuid %s' % uuid)
+        LOGGER.debug('No spider_info given for uuid %s' % uuid)
         self.chan.basic_ack(delivery_tag=delivery_tag)
         return None
     
@@ -243,21 +257,21 @@ class WorkerServer(BaseServer):
         account = account_info[0]
         function_name = spider_info[0]['type']
         if self.service_mapping and self.service_mapping.has_key(function_name):
-            LOGGER.info('Remapping resource %s to %s' % (function_name, self.service_mapping[function_name]))
+            LOGGER.debug('Remapping resource %s to %s' % (function_name, self.service_mapping[function_name]))
             function_name = self.service_mapping[function_name]
         job['function_name'] = function_name
-        # job['reservation_cache'] = None
         job['uuid'] = uuid
         job['account'] = account
         # Save account info in memcached for up to 7 days
         d = self.memc.set(uuid, simplejson.dumps(job), 60*60*24*7)
         d.addCallback(self._createJobCallback, job)
-        d.addErrback(self.workerError)
+        d.addErrback(self.workerErrback, 'Create Job')
         return d
     
     def _createJobCallback(self, memc, job):
         return job
-        
+    
+    # TODO: map before putting into memcache
     def mapKwargs(self, job):
         kwargs = {}
         service_name = job['function_name'].split('/')[0]
